@@ -61,6 +61,11 @@
 #include "optimizer/var.h"
 #else
 #include "executor/tuptable.h"
+#include "optimizer/appendinfo.h"
+#endif
+
+#ifdef PACKAGE_URL
+#undef PACKAGE_URL
 #endif
 
 /* GDAL/OGR includes and compat */
